/* 
 * CuImage, a CUDA image processing class library
 * Copyright (C) 2007 Wouter Caarls, MPIbpc
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __CUIMAGE_H_INCLUDED
#define __CUIMAGE_H_INCLUDED

#include <cmm.h>

#ifdef DLLIMPORTEXPORT
#undef DLLIMPORTEXPORT
#endif

#ifdef CUIMAGE_DLL
#define DLLIMPORTEXPORT __declspec(dllexport)
#else
#define DLLIMPORTEXPORT __declspec(dllimport)
#endif

class DLLIMPORTEXPORT CuImage : public cmm::cmmPagedMemory
{
public:
  CuImage() : cmm::cmmPagedMemory(cmm::cmmDims()) {}
  explicit CuImage(const cmm::cmmDims &dims) : cmm::cmmPagedMemory(dims) {}
  CuImage(CuImage &img);
  CuImage &operator=(CuImage &rhs);

  CuImage operator()(int xshift, int yshift);
  CuImage operator()(int xstart, int xend, int ystart, int yend);

	CuImage &SetDataInt8(const unsigned char *const data);
	unsigned char *GetDataInt8(void);
	CuImage &SetDataInt16(const unsigned short *const data);
	unsigned short *GetDataInt16(void);

  DLLIMPORTEXPORT friend CuImage operator+(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator-(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator*(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator/(CuImage &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage operator>(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator>=(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator<(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator<=(CuImage &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage operator==(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator||(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator&&(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator|(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator&(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator^(CuImage &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage fmax(CuImage &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage fmin(CuImage &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage operator+(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator-(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator*(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator/(CuImage &lhs, const float &rhs);

  DLLIMPORTEXPORT friend CuImage operator>(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator>=(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator<(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator<=(CuImage &lhs, const float &rhs);

  DLLIMPORTEXPORT friend CuImage operator==(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator||(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator&&(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator|(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator&(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage operator^(CuImage &lhs, const float &rhs);

  DLLIMPORTEXPORT friend CuImage fmax(CuImage &lhs, const float &rhs);
  DLLIMPORTEXPORT friend CuImage fmin(CuImage &lhs, const float &rhs);

  DLLIMPORTEXPORT friend CuImage operator+(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator-(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator*(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator/(const float &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage operator>(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator>=(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator<(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator<=(const float &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage operator==(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator||(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator&&(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator|(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator&(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage operator^(const float &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage fmax(const float &lhs, CuImage &rhs);
  DLLIMPORTEXPORT friend CuImage fmin(const float &lhs, CuImage &rhs);

  DLLIMPORTEXPORT friend CuImage sqrt(CuImage &img);
  DLLIMPORTEXPORT friend CuImage log(CuImage &img);
  DLLIMPORTEXPORT friend CuImage abs(CuImage &img);
  DLLIMPORTEXPORT friend CuImage operator!(CuImage &img);

  DLLIMPORTEXPORT friend float sum(CuImage &img);
  DLLIMPORTEXPORT friend float fmin(CuImage &img);
  DLLIMPORTEXPORT friend float fmax(CuImage &img);
  DLLIMPORTEXPORT friend float mean(CuImage &img);
  DLLIMPORTEXPORT friend float var(CuImage &img);

  DLLIMPORTEXPORT friend CuImage erosion(CuImage &img, int iterations=1, int connectivity=0);
  DLLIMPORTEXPORT friend CuImage dilation(CuImage &img, int iterations=1, int connectivity=0);
  DLLIMPORTEXPORT friend CuImage gauss(CuImage &img, float sigma);
  DLLIMPORTEXPORT friend CuImage gaussx(CuImage &img, float sigma);
  DLLIMPORTEXPORT friend CuImage gaussy(CuImage &img, float sigma);
  DLLIMPORTEXPORT friend CuImage convolve(CuImage &img, float *kernel, int xradius, int yradius);

  DLLIMPORTEXPORT friend CuImage slice(CuImage &img, float xstart, float ystart, int width, int height);
  DLLIMPORTEXPORT friend CuImage affine(CuImage &img, float *a);
  DLLIMPORTEXPORT friend CuImage affine(CuImage &img, float a11, float a12, float a13, float a21, float a22, float a23);
  DLLIMPORTEXPORT friend CuImage xx(CuImage &img);
  DLLIMPORTEXPORT friend CuImage yy(CuImage &img);

  DLLIMPORTEXPORT friend void fft(CuImage &img, CuImage &real, CuImage &imag);
  DLLIMPORTEXPORT friend CuImage ifft(CuImage &real, CuImage &imag);

  DLLIMPORTEXPORT friend CuImage merge3(CuImage &in1, CuImage &in2, CuImage &in3);
  DLLIMPORTEXPORT friend void split3(CuImage &in, CuImage &out1, CuImage &out2, CuImage &out3);

  DLLIMPORTEXPORT friend CuImage resize(CuImage &img, const cmm::cmmDims &dims, int xstart=0, int ystart=0);

  DLLIMPORTEXPORT friend void histogram256(CuImage &img, unsigned int *out);
  DLLIMPORTEXPORT friend CuImage label(CuImage &img);
};

#endif /* __CUIMAGE_H_INCLUDED */
